def extract_date_wildlife_acoustics_filename(filepath: str) -> str :
    pass

def extract_metadata(filepath:str) -> dict :
    pass

def update_metadata(filepath: str, metadata: dict) -> dict :
    pass
